"""
Absfuyu: Development
---
Some stuffs that are not ready to use yet

Version: 2.0.0
Date updated: 23/11/2023 (dd/mm/yyyy)
"""


# Module level
###########################################################################
__all__ = [
    "password_check", "fib",
]


# Library
###########################################################################
import re
import os as __os
from functools import lru_cache



# PASSWORD CHECKER
def password_check(password: str) -> bool:
    """
    Verify the strength of 'password'.
    Returns a dict indicating the wrong criteria.
    A password is considered strong if:
    - 8 characters length or more
    - 1 digit or more
    - 1 symbol or more
    - 1 uppercase letter or more
    - 1 lowercase letter or more
    """

    # calculating the length
    length_error = len(password) < 8

    # searching for digits
    digit_error = re.search(r"\d", password) is None

    # searching for uppercase
    uppercase_error = re.search(r"[A-Z]", password) is None

    # searching for lowercase
    lowercase_error = re.search(r"[a-z]", password) is None

    # searching for symbols
    symbols = re.compile(r"[ !#$%&'()*+,-./[\\\]^_`{|}~"+r'"]')
    symbol_error = symbols.search(password) is None

    detail = {
        'password_ok': not any([ # overall result
            length_error, digit_error,
            uppercase_error, lowercase_error,
            symbol_error
        ]),
        'length_error': length_error,
        'digit_error': digit_error,
        'uppercase_error': uppercase_error,
        'lowercase_error': lowercase_error,
        'symbol_error': symbol_error,
    }

    return detail['password_ok']



# FIBONACCI WITH CACHE
@lru_cache(maxsize=5)
def fib(n: int) -> int:
    """Fibonacci (recursive)"""
    # max recursion is 484
    if n < 2:
        return n
    return fib(n-1) + fib(n-2)



# https://stackoverflow.com/questions/563022/whats-python-good-practice-for-importing-and-offering-optional-features
def optional_import(module: str, name: str = None, package: str = None):
    import importlib
    try:
        module = importlib.import_module(module)
        return module if name is None else getattr(module, name)
    except ImportError as e:
        if package is None:
            package = module
        msg = f"install the '{package}' package to make use of this feature"
        import_error = e

        def _failed_import(*args):
            raise ValueError(msg) from import_error

        return _failed_import
